"use strict";
/**
 *  lambda-lambda-lambda/cli
 *  Command-line tool to create a new L³ application.
 *
 *  Copyright 2023-2024, Marc S. Brooks (https://mbrooks.info)
 *  Licensed under the MIT license:
 *  http://www.opensource.org/licenses/mit-license.php
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFiles = createFiles;
exports.createFile = createFile;
const change_case_1 = require("change-case");
const child_process_1 = require("child_process");
const template_file_1 = require("template-file");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
/**
 * Generate app sources from templates.
 */
async function createFiles(appConfig, outPath) {
    const templates = getTemplatePath();
    const manifest = `${templates}/MANIFEST`;
    const vars = {
        appName: (0, change_case_1.camelCase)(appConfig.name),
        appDescription: appConfig.description,
        appPrefix: appConfig.prefix,
        appTimeout: appConfig.timeout,
        appRuntime: appConfig.runtime,
        pkgName: (0, change_case_1.paramCase)(appConfig.name),
        sdkPackage: (appConfig.sdkVersion === '2') ? 'aws-sdk-mock' : 'aws-sdk-client-mock',
        cfResourceName: (0, change_case_1.pascalCase)(appConfig.name),
        routePath: '/example',
        routePrefix: ((appConfig.prefix !== '/') ? appConfig.prefix : ''),
        nodeVersion: appConfig.runtime.replace(/[^0-9]/g, '')
    };
    const manFiles = (await (0, template_file_1.renderFile)(manifest, { ...vars })).split(/\r?\n/);
    const tplFiles = fs.readdirSync(templates);
    for (let tplFile of tplFiles) {
        let outFile = getFsPath(manFiles, tplFile, outPath);
        if (outFile) {
            const outDir = path.dirname(outFile);
            // TODO: Remove VS Code extension pass-thru (e.g. Yes value)
            const isAsync = (appConfig.asynchronous === true || appConfig.asynchronous === 'Yes');
            // Select template based on type.
            if (isMiddleware(outFile)) {
                tplFile = `${templates}/${path.basename(outFile)}`;
            }
            else if (isRoute(outFile)) {
                tplFile = `${templates}/route` + fileSuffix(isAsync) + '.js';
            }
            else if (isApp(outFile)) {
                tplFile = `${templates}/app` + fileSuffix(isAsync) + '.js';
            }
            else {
                tplFile = `${templates}/${tplFile}`;
            }
            outFile = outFile.replace(/-async/, '');
            if (!fs.existsSync(outDir)) {
                fs.mkdirSync(outDir, { recursive: true });
            }
            const content = await (0, template_file_1.renderFile)(tplFile, { ...vars });
            fs.writeFileSync(outFile, content, 'utf8');
        }
    }
}
/**
 * Generate file source from a template.
 */
async function createFile(name, outPath, basePath) {
    const templates = getTemplatePath();
    let resPath = getResourcePath(outPath);
    resPath = (resPath) ? `${resPath}/` : '';
    const vars = {
        routePath: `${getAppPrefix(basePath)}/${resPath}${name.toLowerCase()}`
    };
    const outFile = `${outPath}/${(0, change_case_1.pascalCase)(name)}.js`;
    // Select template based on type.
    const tplFile = (isMiddleware(outPath))
        ? `${templates}/middleware.js`
        : `${templates}/route.js`;
    const content = await (0, template_file_1.renderFile)(tplFile, { ...vars });
    fs.writeFileSync(outFile, content, 'utf8');
}
/**
 * Return configured application prefix.
 */
function getAppPrefix(basePath) {
    const files = require('find').fileSync('config.json', basePath);
    if (files) {
        return JSON.parse(fs.readFileSync(files[0], 'utf8')).router.prefix;
    }
    throw new Error('Failed to load application config');
}
/**
 * Return output path for a given file.
 */
function getFsPath(files, cmpFile, outPath) {
    if (outPath && cmpFile && outPath) {
        cmpFile = cmpFile.replace(/^\_/, '.');
        for (const file of files) {
            const regex = new RegExp(`\/${path.parse(cmpFile).name}`);
            if (regex.test(file) || path.basename(file) === cmpFile) {
                return `${outPath}/${file}`;
            }
        }
    }
}
/**
 * Return global templates source path.
 */
function getTemplatePath() {
    if (process.env.NODE_ENV === 'production') {
        const moduleLib = (0, child_process_1.execSync)('npm root --location=global --loglevel=error').toString().trim();
        return `${moduleLib}/@lambda-lambda-lambda/cli/templates`;
    }
    return `${__dirname}/../templates`;
}
/**
 * Return path relative to resource directory.
 */
function getResourcePath(value) {
    return value.replace(/^.*\/src\/(?:middleware|routes)\/?(.*)?$/, '$1');
}
/**
 * Check for middleware output path.
 */
function isMiddleware(path) {
    return !!/\/src\/middleware/.test(path);
}
/**
 * Check for route output path.
 */
function isRoute(path) {
    return !!/\/src\/routes/.test(path);
}
/**
 * Check for app output path.
 */
function isApp(path) {
    return !!/\/src\/app/.test(path);
}
/**
 * Return -async file suffix.
 */
function fileSuffix(add) {
    return add ? '-async' : '';
}
//# sourceMappingURL=generator.js.map